// Basic interface I/O for plugin.
#include "PluginBase/SCMDPlugin.h"
#include <stdio.h>



const char* PluginName = "Test trigger plugin"; // Plugin menu name



// This function is called when the plugin is being initialized.
void Initialize() {
	/* Your code here */


	//<---- Example code. Delete this.
	freopen("output.txt", "w", stdout);
	setvbuf(stdout, NULL, _IONBF, 0);
	//----> Example code. Delete this.
}




// This function is called when the DLL is unloaded.
void Finalize() {
	/* Your code here */


	//<---- Example code. Delete this.
	fclose(stdout);
	//----> Example code. Delete this.
}



// This code is run when the menu is pressed.
BOOL WINAPI RunPlugin(	TEngineData*	EngineData,		//	Struct containing engine data
						DWORD CurSection,				//	Section plugin is being run for (Currently either triggers or mission briefing)
						CChunkData*	Triggers,			//	Pointer to trigger datachunk
						CChunkData*	MissionBriefing,	//	Pointer to mission briefing datachunk
						CChunkData*	SwitchRenaming,		//	Pointer to switch renaming datachunk
						CChunkData*	UnitProperties,		//	Pointer to unit properties datachunk
						CChunkData*	UnitPropUsage	)	//	Pointer to unit property usage datachunk
{
	// If any of required chunks are not present, then just return.
	if ((Triggers == NULL)||(MissionBriefing == NULL)||(SwitchRenaming == NULL)||(UnitProperties == NULL)||(UnitPropUsage == NULL))
		return FALSE; // Plugin process failed.

	
	if (CurSection == 'GIRT') {
		/* Put your code here*/


		//<---- Example code. Delete this.
		SI_VirtSCStringList* stb = EngineData->MapStrings; // String table we'll be using for.


		printf("========== String Table Test ==========\n");

		// We're going to modify string table a bit, but surely we won't want to mess up with our original map.
		// StringTable_BackupStrings backups string table.
		StringTable_BackupStrings(stb);


		int strn = StringTable_GetTotalStringNum(stb); // Total number of string entries.
		printf("Total string number = %d\n", strn);
		// StringTable_GetTotalStringNum = #(Used strings) + #(Unused string entries)
		// So used strings are really fewer than strn.


		// StringTable_GetString retrives string of some string ID.
		const char* str = StringTable_GetString(stb, 10); // Get string #10. String counts from #0.
		printf("%s\n", str);


		// Find the string we've retrived from string table.
		int findindex = StringTable_FindString(stb, str);
		printf("(may be different)    %d %d\n", findindex, 10);

		// String table may have duplicate strings, so findindex could be different string ID having the same content as string #10.
		// But stringtable[findindex] should be the same as stringtable[10].

		const char* str2 = StringTable_GetString(stb, findindex);
		printf("(should be 0)         %d\n", strcmp(str, str2));


		// AddString test.
		//  AlwaysCreate = 0 -> Duplicate strings may be recycled.
		int newindex = StringTable_AddString(stb, str, 'GRIT', 0); // recycle string.
		printf("(should be the same)  %d %d\n", findindex, newindex);

		// As AddString returns string index for existing string, it automatically adds reference to it.
		// Revert the change.
		StringTable_Dereference(stb, newindex, 'GRIT', 0); //deref


		// AddString test.
		//  AlwaysCreaate = 1 -> AddString won't recycle existing string; AddString always create new string.
		//  AddString may use existing empty entries.
		newindex = StringTable_AddString(stb, str, 'GRIT', 1); // create new string
		printf("(should be different) %d %d\n", findindex, newindex); // No string recycling, so two numbers should be different

		// P.S) GetString won't add reference to its returned string ID.
		str2 = StringTable_GetString(stb, newindex);
		printf("%s\n", str2); // should print the same as str.

		// Remove reference from newindex.
		// Since newindex points to some newly created string, it's reference count should be 1 by here.
		// Deleting reference from newindex will make its reference count 0.
		// If a string gets 0 reference count, the string is removed.
		StringTable_Dereference(stb, newindex, 'GRIT', 0); // This should delete our newly created string

		// Deleted string have NULL as their content.
		str2 = StringTable_GetString(stb, newindex);
		printf("(should be (null))    %s\n", str2);


		// Total string dump.
		printf("Total dump:\n");
		for(int i = 0 ; i < strn ; i++) {
			printf("%d : %s\n", i, StringTable_GetString(stb, i));
		}


		// Restore string table backup. All the things we've done above is nullified.
		StringTable_RestoreBackup(stb);


		// Default Unit name dump. "Terran Marine" things.
		printf("========== Default Unit name dump ==========\n");
		for(int i = 0 ; i < 228 ; i++) {
			printf("%d : %s\n", i, EngineData->UnitNames[i]);
		}

		// Custom Unit name dump.
		printf("========== Custom unit name dump ==========\n");
		for(int i = 0 ; i < 228 ; i++) {
			// Custom unit names are stored with string index.
			int unitstrindex = EngineData->UnitCustomNames[i];
			if(unitstrindex == 0) continue; // custom name is not specified with the unitid.
			printf("%d : %s\n", i, StringTable_GetString(stb, EngineData->UnitCustomNames[i]));
		}

		// Location dump.
		printf("========== Location dump ==========\n");
		for(int i = 0 ; i < 255 ; i++) {
			LocationNode& loc = EngineData->MapLocations[i];
			if(!loc.Exists) continue;
			printf("Location %d:\n"
				"  x0 : %d\n"
				"  y0 : %d\n"
				"  x1 : %d\n"
				"  y1 : %d\n"
				"  Name : %s\n"
				"  Elevation : 0x%04X\n"
				"\n", i, 
				loc.Data.x0, loc.Data.y0, loc.Data.x1, loc.Data.y1,
				StringTable_GetString(stb, loc.Data.Name), loc.Data.Elevation
			);
		}

		//----> Example code. Delete this.

	}
	return TRUE;
}



